//*****************************************************************************
// Description     : Main sample program of histogram mode
// Target Device   : APV8016A
// Target Platform : Linux, C
// Date            : Jan. 2021
// Version         : 1.0
//*****************************************************************************
#include <stdio.h>
#include <string.h>

#include "config.h"
#include "file.h"
#include "sitcp.h"
#include "device.h"
#include "local_define.h"

//
// Local functions declaration
//
static int run_hist_measurement(int, int, unsigned qword);
static int receive_hist(int, int);


//#########################################################################
//  Name      : main
//  Function  : entry point
//  Arguments :
//      in argc : number of parameters
//      in argv : pointer to array of parameters
//  Return :
//      0 : success
//      1 : failure
//#########################################################################
int main(int argc, char **argv)
{
	int command_socket, data_socket;
	int ret;
	int run_ready = 0;
	unsigned qword meas_count;

	// 0. delete data file if exists
	unlink(HIST_FILENAME);

	// 1. connect to the device via SiTCP (both command and data)
	ret = sitcp_cmd_init(DEVICE_PORT_CTRL_OWNPC, &command_socket);
	if (ret < 0) {
		printf("connect command error(%d)\n", ret);
		return(1);
	}
	ret = sitcp_data_init(DEVICE_IPADDR, &data_socket);
	if (ret < 0) {
		printf("connect data error(%d)\n", ret);
		return(2);
	}

	// 2. set parameters to the device
	meas_count = device_msec2count(MEASUREMENT_TIME_MSEC);	// convert msec to count
	ret = config_device(command_socket, meas_count, DEVICE_PARAM_MOD_HIST);
	if (ret < 0) {
		printf("device config error(%d)\n", ret);
	}
	else {
		run_ready = 1;
	}

	if (run_ready == 1) { // ready to run
		// 3. start measurement
		printf("start measurement!!!\n");
		ret = run_hist_measurement(command_socket, data_socket, meas_count);
		if (ret < 0) {
			printf("save histogram data error(%d)\n", ret);
		} else {
			printf("save histogram data finished\n");
		}

		// 4. stop measurement
		sitcp_write_16bit(command_socket, DEVICE_COMMAND_AQS, DEVICE_PARAM_AQS_STOP);
		printf("stop measurement!!!\n");
	}

	// 5. disconnect from device
	close(command_socket);
	close(data_socket);

	return(0);
}


//#########################################################################
//  Name      : run_hist_measurement
//  Function  : histogram measurement processing
//  Arguments :
//      in sock_cmd : file descriptor for command communication
//      in sock_dat : file descriptor for data transfer
//      in ms_count : measurement time (counts)
//  Return :
//       0 : success
//      -1 : failed to write file header
//      -11,-12 : failed to receive histogram data; return value of receive_hist()-10
//#########################################################################
static int run_hist_measurement(int sock_cmd, int sock_dat, unsigned qword ms_count)
{
	int ret;
	unsigned int rtm_msec;
	unsigned qword rtm_count;
	unsigned short status;
	unsigned int input_cps[DEVICE_MAX_CHNUM];
	unsigned int throughput_cps[DEVICE_MAX_CHNUM];

	// clear data
	sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_CLR, 0);
	sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_CLR, 1);
	sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_CLR, 0);

	// wait for completion of clear processing
	msleep(200);

	// start measurement(issue stop command first, in case of in the middle of measurement)
	sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_AQS, DEVICE_PARAM_AQS_STOP);
	sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_AQS, DEVICE_PARAM_AQS_START);

	// display top 2 lines of status field to explain each column(first 4CHs only as example)
	printf("err  <tm(ms)>|        <input rate(cps) / throughput rate(cps)>\n");
	printf("cnt    real  |     CH1     |     CH2     |     CH3     |     CH4\n");
	printf("-------------+-------------+-------------+-------------+-------------\n");

	while(1) {
		// read status
		ret = device_read_status(sock_cmd, 4, &status, &rtm_count, input_cps, throughput_cps);
		if (ret > 0) {
			printf("\n");
			fflush(stdout);
		}
		// status is 'stopped' or measurement time elapsed
		if ((status == DEVICE_PARAM_AQS_STOP) || (rtm_count >= ms_count)) {
			break;
		}

		msleep(500);	// wait for a while, as you like
	}

	// write file header
	rtm_msec = (unsigned int)(device_count2msec(rtm_count));	// convert count to msec
	ret = file_write_text_header(HIST_FILENAME, rtm_msec,
		   						DEVICE_MAX_CHNUM, input_cps, throughput_cps);
	if (ret < 0) {
		return(-1);
	}

	// receive last histogram data then write out to file
	ret = receive_hist(sock_cmd, sock_dat);
	if (ret < 0) {
		return(-10+ret);
	} else {
		return(0);
	}
}


//#########################################################################
//  Name      : receive_hist
//  Function  : receiving histogram data
//  Arguments :
//      in sock_cmd : file descriptor for command communication
//      in sock_dat : file descriptor for data transfer
//  Return :
//      positive value : receivied data size
//      -1 : failed to receive
//      -2 : failed to write file
//#########################################################################
static int receive_hist(int sock_cmd, int sock_dat)
{
	int ret, i;
	int result = 0;
	unsigned int hdata[DEVICE_MAX_CHNUM][NUM_DATA];

	memset(hdata, 0, sizeof(hdata));
	for (i = 0; i < DEVICE_MAX_CHNUM; i++) {
		// receiving histogram data
		ret = device_read_histogram(sock_cmd, sock_dat,
								   (char*)(&hdata[i]), sizeof(hdata[i]), i);
	}

	if (ret > 0) {
		// write data to file
		ret = file_write_text_data(HIST_FILENAME, DEVICE_MAX_CHNUM, hdata);
		if (ret < 0) {
			// failed to write
			result = -2;
		}
	} else {
		// failed to receive data
		result = -1;
	}

	return(result);
}
