//*****************************************************************************
// Description     : Definition of functions for controlling device
// Target Device   : APV8016A
// Target Platform : Linux, C
// Date            : Nov. 2020
// Version         : 1.0
//*****************************************************************************
#include <stdio.h>
#include <arpa/inet.h>

#include "sitcp.h"
#include "device.h"


//#########################################################################
//  Name      : device_read_status
//  Function  : read some status information from device
//  Arguments :
//      in  sock_cmd  : file descriptor for command communication
//      in  disp_chnum: number of CHs to display rate
//      out pStat     : pointer to the measurement status
//      out pTime     : pointer to the real time
//      out pInputCps : pointer to the input count rate
//      out pThrouCps : pointer to the throughput count rate
//  Return :
//       1 : success
//       0 : success but no valid information
//  Note : For the first 1sec after measuremnt starts,
//         status information shall be ignored
//         since the device is not completety initialized yet.
//#########################################################################
int device_read_status(int sock_cmd, int disp_chnum,
					   unsigned short *pStat, unsigned qword *pTime,
					   unsigned int *pInputCps, unsigned int *pThrouCps)
{
	int ret, i;
	unsigned qword real_time;
	int err = 0;

	//
	// read each status
	// (read all data before display
	// in order to obtain values at the same possible point in time)
	//

	// measurement status: 2byte value(receive 16bit length data)
	*pStat = sitcp_read_16bit(sock_cmd, DEVICE_COMMAND_AQS, &ret);
	if (ret != 0) {
		err++;
		*pStat = 10;	// set non-zero not to stop measurement at minor error
	}

	// input count rate: 4byte value(receive 32bit length data)
	for (i = 0; i < DEVICE_MAX_CHNUM; i++) {
		*(pInputCps + i) = sitcp_read_32bit(sock_cmd, DEVICE_STATUS_ICR + DEVICE_OFFSETADDR_CH * i, &ret);
		if (ret != 0) {
			err++;
			*(pInputCps + i) = 11;	// error mark
		}
	}

	// throughput count rate: 4byte value(receive 32bit length data)
	for (i = 0; i < DEVICE_MAX_CHNUM; i++) {
		*(pThrouCps + i) = sitcp_read_32bit(sock_cmd, DEVICE_STATUS_TCR + DEVICE_OFFSETADDR_CH * i, &ret); 
		if (ret != 0) {
			err++;
			*(pThrouCps + i) = 22;	// error mark
		}
	}

	// real time: 6byte value(receive 48bit length data)
	*pTime = sitcp_read_48bit(sock_cmd, DEVICE_STATUS_RTM, &ret);
	if (ret != 0) {
		err++;
		*pTime = 1011;	// error mark, should be over 1000 to display at callee
	}

	//
	// display
	//
	real_time = (unsigned qword)(device_count2msec(*pTime));	// convert count to msec
	if (real_time >= 1000) {
		if (err > 0) {
			printf("%2d %9llu", err, real_time);
		} else {
			printf("   %9llu", real_time);
		}
		for (i = 0; i < disp_chnum; i++) { // display only disp_chnum of CHs as example
			printf(" | %5d/%5d", *(pInputCps+i), *(pThrouCps+i));
		}
		return(1);
	} else {
		// for the first 1sec, information shall be ignored
		return(0);
	}
}


//#########################################################################
//  Name      : device_read_histogram
//  Function  : read histogram data from device
//  Arguments :
//      in  sock_cmd : file descriptor for command communication
//      in  sock_dat : file descriptor for data taransfer
//      out rbuffer : pointer to the data buffer
//      in  nbyte   : request data size(byte)
//      in  chidx   : CH index(0=CH1, 1=CH2, ...)
//  Return :
//      positive value : received data size
//      -1 : failed to data request
//      -2 : failed to data receive
//  Note : This function converts histogram data from network byte order
//         to host byte order
//#########################################################################
int device_read_histogram(int sock_cmd, int sock_dat,
						  char *rbuffer, int nbyte, int chidx)
{
	int i, unit;
	int ret;
	int result;
	int read_byte, recv_byte = 0;
	int recv_cnt = 0;
	unsigned int *p;

	result = 0;

	// request histogram data of CH(specified by chidx)
	ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_RQH, chidx);

	if (ret >= 0) { // succeeded to request histogram data
		msleep(10);

		while(1) {
			// update request data size
			read_byte = nbyte - recv_byte; 

			// receive histogram data via port for fast data transfer
			ret = recv(sock_dat, &rbuffer[recv_byte], read_byte, MSG_DONTWAIT);
			if (ret <= 0) {
				// no data or receive error
				break;
			}

			// update received data size
			recv_byte += ret;

			if (recv_byte >= nbyte) {
				// finish when received data size reached to request one
				break;
			}

			// update recv count
			recv_cnt++;
			if (recv_cnt > 100) break;	// faile safe for avoiding infinite loop
		}

		if (recv_byte > 0) { // succeeded to receive
			// set total received size
			result = recv_byte;

			// convert histogram data from network byte order to host byte order
			p = (unsigned int*)(rbuffer);
			unit = nbyte / sizeof(unsigned int);

			for(i = 0; i < unit; i++) {
				p[i] = ntohl(p[i]);
			}
		}
		else {// failed to receive
			result = -2;
		}
	}
	else { // failed to request
		result = -1;
	}

	return(result);
}
